# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require 'singleton'
require 'yaml'

module NihongoBenkyo

    module Config
    
        if RUBY_PLATFORM =~ /win32/
            USER_DIR = File.join(ENV['USERPROFILE'],"nihongobenkyo")
        else
            USER_DIR = File.join(ENV['HOME'],".nihongobenkyo")
        end

        # The dir that holds the config file and the dic.db file
        # is eitheir in a shared data dir or in a personal data dir
        if !FileTest.exists?(File.join(USER_DIR,"dic.db")) \
           and FileTest.exists?(File.join(DATA_DIR,"dic.db"))
            CONFIG_DIR = DATA_DIR
        else
            CONFIG_DIR = USER_DIR
        end
    	
        DB_FILE  = File.join(CONFIG_DIR,"dic.db")
        CONFIG_FILE = File.join(CONFIG_DIR,"config.yaml")
        DEFAULT_CONFIG_FILE = File.join(DATA_DIR, "config", "default.yaml")
    end

    class Preferences
        include Singleton
    
        def initialize
            unless(FileTest.exists?(Config::CONFIG_DIR))
                Dir.mkdir(Config::CONFIG_DIR)    
            end
            
            unless(FileTest.exists?(Config::DB_FILE))
                File.new(Config::DB_FILE, File::CREAT).close
            end
            
            if(FileTest.exists?(Config::CONFIG_FILE))
                @config = YAML.load(File.open(Config::CONFIG_FILE))
            else
                @config = YAML.load(File.open(Config::DEFAULT_CONFIG_FILE))
            end
        end
        
        def save!
            File.open(Config::CONFIG_FILE, File::CREAT|File::TRUNC|File::RDWR, 0600) do |f|
                YAML.dump(@config, f)
            end
        end
        
        def method_missing(id, *args)
            method = id.id2name
            if match = /(.*)=$/.match(method)
                if args.length != 1
                    raise "Set method #{method} should be called with " +
                          "only one argument (was called with #{args.length})"
                end
                @config[match[1]] = args.first
            else
                unless args.empty?
                    raise "Get method #{method} should be called " +
                          "without argument (was called with #{args.length})"
                end
                @config[method]
            end                
        end
            
    end

end
