# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo
    class Radical 
    
        def self.parse_file
            path = File.join(NihongoBenkyo::Config::DATA_DIR, 
                             "radicals", "radkfile_utf8")
            f = File.new(path, "r")
    
            rad = nil
            strokes = nil
            sense = nil
            kanji = nil
    
            f.each_line do |line|
                next if line =~ /^#/
                line.chomp!
                if line =~ /^\$/
                    yield kanji, rad, strokes, sense unless kanji.nil?
                    rad, strokes, sense = line.split(" ")[1,3]
                    kanji = ""
                else
                    kanji += line
                end 
            end
            yield kanji, rad, strokes, sense
        end
    
        def self.list
            radlist = []
            parse_file do |kanji, rad, strokes, sense|
                radlist << rad
            end
            radlist
        end
    
        def self.with_nb_strokes(n)
            radlist = []
            parse_file do |kanji, rad, strokes, sense|
                radlist << rad if strokes.to_i == n
            end
            radlist    
        end
    
        def self.of_kanji(k)
            radlist = []
            parse_file do |kanji, rad, strokes, sense|
                radlist << "#{rad} #{sense}" \
                    if kanji.split("").include?(k)
            end
            radlist   
        end
    
        def initialize(rad)
            @rad = rad
        end
    
        def kanji_list
            klist = []
            Radical.parse_file do |kanji, rad, strokes, sense|
                klist = kanji.split("") if rad == @rad
            end
            klist    
        end
    
    end
end

if __FILE__ == $0
    # N.B. Execute this file with ruby -Ku    
    require 'nihongobenkyo/config'
    
    puts "Characters with radical 女"
    puts NihongoBenkyo::Radical.new("女").kanji_list.join(",")
    puts "Radicals with 3 strokes"
    puts NihongoBenkyo::Radical.with_nb_strokes(3).join(",")
    puts "Radicals of the character 勉"
    puts NihongoBenkyo::Radical.of_kanji("勉").join(",")

end