# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo

module Icon
    icons_dir = File.join(Config::DATA_DIR, "icons")
    LOGO = Gdk::Pixbuf.new(File.join(icons_dir, "nihongobenkyo.png"))
    RESULT_LIST = Gtk::Stock::JUSTIFY_FILL
    RESULT_DETAILS = Gtk::Stock::ZOOM_IN
end

module UI

    class MainApp < GladeBase 
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")
        
        module Page
            RESULT_LIST, RESULT_DETAILS = (0..1).to_a
        end

        def initialize
            super("main_app.glade")
            @prefs = Preferences.instance
            @db = Database.instance
            
            initialize_ui
            initialize_signals
            load_preferences
        end
   
        private
        
        #################### Search methods ####################
        
        def search(search)
            
            unless search.has_key?(:dicname)
                search[:dicname] = active_dictionary    
            end
            
            activate_dictionary(search[:dicname])
            
            dic = Dictionary.new(search[:dicname])

            if search[:entry_id].nil?
                # Search from a word            
                search[:word].clean!
                @search_entry.text = search[:word]
                
                results = dic.search_result_list(search[:word])
                search[:nb_res] = results.length
                                       
                if search[:nb_res] == 1
                    search[:entry_id] = results.first.id
                elsif search[:nb_res] > 1    
                    search[:res_type] = ResultType::LIST
                    @notebook.page = Page::RESULT_LIST
                    @result_listview.results = results 
                end
            end
            
            unless search[:entry_id].nil?
                # Search details from an entry id
                search[:res_type] = ResultType::DETAILS
                search[:nb_res] = 1
                @notebook.page = Page::RESULT_DETAILS
                
                case dic.type
                    when DicType::WORD
                        search[:word] = search_word_details(search[:entry_id])
                    when DicType::KANJI
                        search[:word] = search_kanji_details(search[:entry_id])
                end        
            end

            # Status bar
            if search[:nb_res] > 1
                message = _("Matches found for %s: %d") % [search[:word], search[:nb_res]]
            elsif search[:nb_res] == 1
                message = _("Informations about: %s") % search[:word]
            else
                message = _("No match found")    
            end
            
            @statusbar.push(0, message)

            # Update pages seen and history
            if @pages_seen.empty? or not(@pages_seen[@current_page] == search)
                update_pages_seen(search)
            end
            @history_listview.update(search) if search[:nb_res] > 0 
        end

        def search_word_details(entry_id)
            @infos_view.buffer.text = ""
            iter = @infos_view.buffer.get_iter_at_offset(0)
            @search_entry.text = ""

            dic = WordDictionary.new(active_dictionary)
            word = dic.search_entry(entry_id)

#             @infos_view.buffer.insert(iter, "* ", "title1") \
#                 if word.important
# 
#             if word.in_kanji.empty?
#                 title = word.readings.join(", ")
#             else
#                 title = word.in_kanji.join(", ")
#             end
#             @infos_view.buffer.insert(iter, title + "\n", "title1")
# 
#             category = [
#                 [word.in_kanji, _("In Kanji")],
#                 [word.readings, _("Readings")]
#             ]
# 
#             category.each do |infos, title2|
#                 @infos_view.buffer.insert(iter, title2 + "\n", "title2")
#                 infos.each do |info|
#                     @infos_view.buffer.insert(iter, info + "\n", "title3")
#                 end
#             end
            
            if(@db.is_important_entry?(entry_id))
                @infos_view.buffer.insert(iter, "* ", "title1")            
            end
            
            title = @db.search_entry_title(entry_id)           
            @infos_view.buffer.insert(iter, title + "\n", "title1")
            
            categories = [
                [_("Kanji writings"), "word_kanji", "word_kanji_info"],
                [_("Readings"), "word_reading", "word_reading_no_kanji",
                                                "word_reading_info",
                                                "word_reading_restriction"],
                [_("Senses"), "word_sense", "word_sense_kanji_restriction",
                                            "word_sense_reading_restriction",
                                            "word_sense_reference",
                                            "word_sense_antonym",
                                            "word_sense_part_of_speech",
                                            "word_sense_field",
                                            "word_sense_misc",
                                            "word_sense_info",
                                            "word_sense_name_type"],
                [_("General informations"), "word_entry", "word_entry_origin_lang",
                                                          "word_entry_dialect",
                                                          "word_entry_etymology"]
            ]
            
            categories.each do |title2, metadata_of, *metadata|
                hash = @db.search_metadata(entry_id, metadata_of, *metadata)
                unless hash.length == 0
                    @infos_view.buffer.insert(iter, title2 + "\n", "title2")    
                    
                    hash.each_pair do |title3, hash2|
                        
                        @infos_view.buffer.insert(iter, title3 + "\n", "title3") 
                        hash2.each_pair do |title4, array|
                            @infos_view.buffer.insert(iter, title4 + "\n", "title4")  
                            array.each do |value|
                                @infos_view.buffer.insert(iter, value + "\n", "value")
                            end
                        end
                    end
                end
            end   
            
            title
        end
        
        def search_kanji_details(entry_id)             
            @infos_view.buffer.text = ""
            iter = @infos_view.buffer.get_iter_at_offset(0)
            @search_entry.text = ""
            
            dic = KanjiDictionary.new(active_dictionary)
            kanji = dic.search_entry(entry_id)
            @infos_view.buffer.insert(iter, kanji.to_s + "\n", "kanji")

            category = [
                [kanji.meanings_by_lang, _("Meanings")],
                [kanji.on_readings, _("On readings")],
                [kanji.kun_readings, _("Kun readings")],
                [Radical.of_kanji(kanji.to_s), _("Multiple radicals")],
                [kanji.strokes, _("Strokes")], 
                [kanji.grade, _("Grade")], 
                [kanji.frequency, _("Frequency")]
            ]
            
            category.each do |infos, title|
                unless infos.empty?
                    @infos_view.buffer.insert(iter, title + "\n", "title2")    
                    @infos_view.buffer.insert(iter, " " + infos.join(", ") + "\n", "title3")
                end
            end

            kanji.to_s
        end
        
        #################### Scan selection method ####################
        
        def scan_selection
            @thread = Thread.new(@search_entry) do |entry|
                
                last_selection = nil
                Gtk::Clipboard.get(Gdk::Selection::PRIMARY).request_text do |clipboard, text|
                    last_selection = text
                end
                
                while true
                    Gtk::Clipboard.get(Gdk::Selection::PRIMARY).request_text do |clipboard, text| 
                        if text != last_selection
                            unless text.nil? or text.empty?                   
                                unless text =~ /^(http|ftp|file)/
                                    last_selection = text
                                    search(:word => text)
                                    @main_app.present 
                                end
                            end
                        end
                    end
                    sleep(0.8)
                end
            end
        end
        
        #################### Load/Save preferences methods ####################

        def load_preferences
            if @prefs.window_maximized
                @main_app.maximize
            else
                @main_app.resize(*@prefs.window_size)
                @main_app.move(*@prefs.window_position) unless @prefs.window_position == [0,0]
            end
            
            @global_actions["Statusbar"].active = @prefs.view_statusbar
            @global_actions["Toolbar"].active = @prefs.view_toolbar
            
            @prefs.last_searches.each do |search|
                @history_listview.append_search(search)
            end
            
            @sidepane.position = @prefs.history_position
            @global_actions["History"].active = @prefs.view_history
            
            search(@prefs.last_search) unless @prefs.last_search.nil?
        end
        
        def save_preferences
            @prefs.view_statusbar = @global_actions["Statusbar"].active?
            @prefs.view_toolbar = @global_actions["Toolbar"].active?
            
            @prefs.last_searches = []
            @history_listview.model.each do |model, path, iter|
                @prefs.last_searches << iter[Column::SEARCH_HASH]
            end
            
            @prefs.history_position = @sidepane.position
            @prefs.view_history = @global_actions["History"].active?
            
            @prefs.window_size = @main_app.size
            @prefs.window_position = @main_app.position
            @prefs.window_maximized = @maximized
            
            @prefs.last_search = @pages_seen[@current_page]
            
            @prefs.save!
        end
        
        #################### Dictionary list ####################
        
        def activate_dictionary(dicname)
            index = nil
            @dictionary_cb.model.each do |model, path, iter|
                index = path if iter[0] == dicname
            end
            @dictionary_cb.active = index.to_str.to_i \
                unless index.nil?
        end

        def active_dictionary
            selected = @dictionary_cb.active
            iter = @dictionary_cb.model.get_iter(selected.to_s)
            iter[0]
        end

        def dictionary_cb_update
            @dictionary_cb.model.clear
            sensitive = false
            Dictionary.list.each do |name, type, selected|
                if selected == 1
                    row = @dictionary_cb.model.append() 
                    row[0] = name
                    sensitive = true
                end
            end
            @dictionary_cb.active = 0
            [@search_button, @search_entry, @dictionary_cb,
             @global_actions["ScanSelection"]].each do |w|
                w.sensitive = sensitive
            end
        end

        def dictionary_menu(word)
            menu = Gtk::Menu.new
            Dictionary.list.each do |name, type, selected|
                if selected == 1
                    item = Gtk::MenuItem.new(_("Search %s" % name))
                    item.signal_connect("activate") do 
                        search(:word => word, :dicname => name)
                    end
                    menu.append(item)
                end
            end
            menu.append(Gtk::SeparatorMenuItem.new)
            item = Gtk::MenuItem.new(_("Copy"))
            item.signal_connect("activate") do |mitem|
                Gtk::Clipboard.get(Gdk::Selection::CLIPBOARD).set_text(word)
            end
            menu.append(item)
            menu.show_all
            menu
        end
        
        #################### Go back/forward methods ####################
        
        def update_pages_seen(search_hash)
            @current_page += 1 unless @pages_seen.length == 0
            @pages_seen[@current_page] = search_hash
            @pages_seen.slice!((@current_page+1..@pages_seen.length-1))
            
            sensitize_go_buttons
        end
        
        def sensitize_go_buttons          
            @global_actions["GoBack"].sensitive = (@current_page == 0) ? false : true
            @global_actions["GoForward"].sensitive = (@current_page == @pages_seen.length - 1) ? false : true 
        end
        
        #################### UI ####################
        
        def initialize_ui
            # Icon
            @main_app.icon = NihongoBenkyo::Icon::LOGO
            
            # History and Result listviews
            
            @history_listview = HistoryListView.new
            @history_listview.show_all
            @history_listview_sw.add(@history_listview)
            
            @result_listview = ResultListView.new
            @result_listview.show_all
            @result_listview_sw.add(@result_listview)
            
            # Informations View
            
            @infos_view = InformationsView.new           
            @infos_view.show_all
            @infos_view_sw.add(@infos_view)
            
            # Entry
            @search_entry.grab_focus
                       
            # Pages seen
            
            @pages_seen = []
            @current_page = 0

            # Go back/forward actions
            
            on_go_back = Proc.new do 
                @current_page -= 1
                search(@pages_seen[@current_page])
                sensitize_go_buttons
            end
            
            on_go_forward = Proc.new do
                @current_page += 1
                search(@pages_seen[@current_page])
                sensitize_go_buttons
            end

            # Global actions : always enabled
            
            on_search = Proc.new { @search_entry.grab_focus }

            #on_add_to_lexicon_manually = Proc.new { }
                        
            on_quit = Proc.new { save_preferences; Gtk.main_quit }
            
            on_preferences = Proc.new do
                set_visible_cols_proc = Proc.new { @result_listview.set_visible_columns }
                
                update_history_proc = Proc.new do |value| 
                    while value < @history_listview.model.nb_rows
                        @history_listview.model.remove_last
                    end

                    # Also update dictionary list
                    dictionary_cb_update
                end 
                
                PreferencesDialog.new(@main_app, set_visible_cols_proc, update_history_proc) 
            end
            
            on_about = Proc.new { AboutDialog.new(@main_app).show }

            # [[name, stock_id, label, accelerator, tooltip, proc], ... ]

            standard_actions = [
                ["NihongoBenkyoMenu", nil, "_Nihongo Benkyo"],
                ["Search", Gtk::Stock::FIND, nil, nil, nil, on_search],
                #["AddToLexiconManually", nil, _("Add to lexicon manually"),
                #     nil, nil, on_add_to_lexicon_manually],
                ["Quit", Gtk::Stock::QUIT, nil, nil, nil, on_quit],
                ["EditMenu", nil, _("_Edit")],
                ["Preferences", Gtk::Stock::PREFERENCES, nil, nil, nil, on_preferences],
                ["ViewMenu", nil, _("_View")],
                ["GoMenu", nil, _("_Go")],
                ["GoBack", Gtk::Stock::GO_BACK, nil, "<alt>Left", nil, on_go_back],
                ["GoForward", Gtk::Stock::GO_FORWARD, nil, "<alt>Right", nil, on_go_forward],
                ["HelpMenu", nil, _("_Help")],
                ["About", Gtk::Stock::ABOUT, _("About"), nil, nil, on_about]
            ]
            
            # Toggle actions
            
            on_toggle_scan_selection = Proc.new do
                if @global_actions["ScanSelection"].active?
                    scan_selection
                else
                    @thread.kill
                end
            end
            
            on_toggle_history = Proc.new do
                @sidepane.child1.visible = @global_actions["History"].active?
            end
            
            on_toggle_statusbar = Proc.new do
                @statusbar.visible = @global_actions["Statusbar"].active?
            end
            
            on_toggle_toolbar = Proc.new do
                @toolbar.visible = @global_actions["Toolbar"].active?
            end
            
            # [[name, stock_id, label, accelarator, tooltip, proc, is_active], ... ]
                   
            toggle_actions = [
                ["ScanSelection", nil, _("Scan selection"), nil, nil, on_toggle_scan_selection, false],
                ["History", nil, _("_History"), "F9", nil, on_toggle_history, true],
                ["Statusbar", nil, _("_Statusbar"), nil, nil, on_toggle_statusbar, true],
                ["Toolbar", nil, _("_Toolbar"), nil, nil, on_toggle_toolbar, true]
            ]
            
            @global_actions = Gtk::ActionGroup.new("Standard actions")
            @global_actions.add_actions(standard_actions)
            @global_actions.add_toggle_actions(toggle_actions)
            
            @global_actions["GoBack"].sensitive = false
            @global_actions["GoForward"].sensitive = false
            
            # Actions enabled when a row is selected
            
            on_copy = Proc.new do
                if @notebook.page == Page::RESULT_LIST
                    iter = @result_listview.selected_iter
                    text = @result_listview.model.get_value(iter, Column::KANJI)
                    text += " 【#{@result_listview.model.get_value(iter, Column::READINGS).gsub("\n",", ")}】"
                    text += @result_listview.model.get_value(iter, Column::MEANINGS).gsub("\n",", ")
                    Gtk::Clipboard.get(Gdk::Selection::CLIPBOARD).set_text(text)
                end
            end
            
            on_see = Proc.new do
                # This allows to trigger the row_activated event
                @result_listview.row_activated(@result_listview.selected_iter.path,\
                    @result_listview.get_column(Column::ENTRY_ID))
            end

            #on_add_to_lexicon = Proc.new { }
            
            row_actions = [
                ["Copy", Gtk::Stock::COPY, nil, nil, nil, on_copy],
                ["See", Gtk::Stock::ZOOM_IN, _("See"),
                    nil, nil, on_see],
                #["AddToLexicon", Gtk::Stock::ADD, _("Add to lexicon"), 
                #    nil, nil, on_add_to_lexicon],
            ]
            
            @row_actions = Gtk::ActionGroup.new("Row actions")
            @row_actions.add_actions(row_actions)
            @row_actions.visible = false
            
            # UI Manager
            
            @uimanager = Gtk::UIManager.new
            @uimanager.insert_action_group(@global_actions, 0)
            @uimanager.insert_action_group(@row_actions, 1)
            @main_app.add_accel_group(@uimanager.accel_group)
            
            ["menus.xml", "toolbar.xml", "popups.xml"].each do |ui_file|
                @uimanager.add_ui(File.join(NihongoBenkyo::Config::DATA_DIR, 
                                            "ui", ui_file))
            end
   
            # Add menu and toolbar to the main window
            menubar = @uimanager.get_widget("/MainMenubar")
            @main_app.child.pack_start(menubar, false).reorder_child(menubar, 0)
            
            @toolbar = @uimanager.get_widget("/Toolbar")
            @toolbar.border_width = 0
            @main_app.child.pack_start(@toolbar, false).reorder_child(@toolbar, 1)
            
            @row_popup = @uimanager.get_widget("/RowPopup")

            # History CB
            @dictionary_cb.model = Gtk::ListStore.new(String)
            dictionary_cb_update

            # Informs the user if there is no dictionary
            if @db.first_time? and !@db.has_dictionaries?
                InfoDialog.new(@main_app, 
                _("There's currently no dictionary configured. You should import one or download a predefined database at nihongobenkyo.org."))
            end        
        end
        
        #################### Signals ####################
        
        def initialize_signals        
            @search_entry.signal_connect("activate") do
                # Perform a search when the user pushes "Enter"
                @search_button.clicked
            end
            
            @search_button.signal_connect("clicked") do
                # Perform a search when the button "Find" is clicked
                search(:word => @search_entry.text)
            end
                   
            @main_app.signal_connect("destroy") do 
                @global_actions["Quit"].activate
            end
            
            @history_listview.selection.signal_connect("changed") do
                # Search from history
                if @history_listview.has_row_selected?
                    search = @history_listview.selected_search
                    
                    if @pages_seen.empty? or not(@pages_seen[@current_page] == search)
                        search(search)
                    end
                end    
            end
            
            @notebook.signal_connect("switch-page") do |nb, page, p_num|
                @result_listview.unselect_all
                @row_actions.visible = false
            end
            
            @result_listview.selection.signal_connect("changed") do
                # Activate the "row actions" when a row is selected
                if @result_listview.has_row_selected?
                    @row_actions.visible = true
                else
                    @row_actions.visible = false    
                end
            end
            
            @main_app.signal_connect("window-state-event") do |w, e|
                # Define the @maximized variable when the window is maximized
                if e.is_a?(Gdk::EventWindowState)
                    @maximized = \
                        e.new_window_state == Gdk::EventWindowState::MAXIMIZED 
                end
            end
            
            @result_listview.signal_connect("row-activated") do |view, path, column|
                # Search when double click on a row (details about an entry)
                if selected_row = view.model.get_iter(path)
                    entry_id = selected_row[Column::ENTRY_ID]
                    search(:entry_id => entry_id)
                end
            end
            
            @result_listview.signal_connect("button_press_event") do |widget, event|
                # Display a popup menu when a row is right-clicked
                if event.kind_of? Gdk::EventButton and event.button == 3
                    @row_popup.popup(nil, nil, event.button, event.time)
                end
            end

            @infos_view.signal_connect("button_press_event") do |widget, event|
                # Display a popup menu when a row is right-clicked
                if event.kind_of? Gdk::EventButton and event.button == 3
                    if @infos_view.buffer.has_selected_text? and \
                       !@global_actions["ScanSelection"].active?
                        sel = @infos_view.buffer.selected_text
                        dictionary_menu(sel).popup(nil, nil, event.button, event.time) 
                    end
                end
            end

            @infos_view.signal_connect("link_clicked") do |widget, word, event|
                dictionary_menu(word).popup(nil, nil, event.button, event.time)   
            end
        end
        
    end
end
end
